@@ -2,10 +2,10 @@ package Chart::Gnuplot;
 use strict;
 use vars qw($VERSION);
 use Carp;
-use File::Copy;
+use File::Copy qw(move);
 use File::Temp qw(tempdir);
-use Chart::Gnuplot::Util qw(_lineType _pointType);
-$VERSION = '0.12';
+use Chart::Gnuplot::Util qw(_lineType _pointType _copy);
+$VERSION = '0.14';
 
 # Constructor
 sub new
@@ -21,8 +21,11 @@ sub new
     }
 
     # Default terminal: postscript terminal with color drawing elements
-    $hash{terminal} = "postscript enhanced color" if
-        (!defined $hash{terminal} && !defined $hash{term});
+    if (!defined $hash{terminal} && !defined $hash{term})
+    {
+        $hash{terminal} = "postscript enhanced color";
+        $hash{_terminal} = 'auto';
+    }
 
     # Default setting
     if (defined $hash{output})
@@ -56,7 +59,7 @@ sub set
     my ($self, %opts) = @_;
     foreach my $opt (keys %opts)
     {
-        $self->$opt($opts{$opt});
+        ($opts{$opt} eq 'on')? $self->$opt('') : $self->$opt($opts{$opt});
     }
     return($self);
 }
@@ -253,8 +256,11 @@ sub _setChart
     if (defined $self->{imagesize})
     {
         my ($ws, $hs) = split(/,\s?/, $self->{imagesize});
-        $ws *= 10 if ($ws =~ /\D/); # for post terminal
-        $hs *= 7 if ($hs =~ /\D/);  # for post terminal
+        if (defined $self->{_terminal} && $self->{_terminal} eq 'auto')
+        {
+            $ws *= 10; # for post terminal
+            $hs *= 7;  # for post terminal
+        }
         $self->{terminal} .= " size $ws,$hs";
     }
 
@@ -462,6 +468,18 @@ sub _setChart
             push(@sets, $attr);
         }
     }
+
+    # Write labels
+    foreach my $label (@{$self->{_labels}})
+    {
+        print PLT "set label $label\n";
+    }
+
+    # Draw arrows
+    foreach my $arrow (@{$self->{_arrows}})
+    {
+        print PLT "set arrow $arrow\n";
+    }
     close(PLT);
 
     $self->_sets(\@sets);
@@ -922,25 +940,24 @@ sub _execute
     # Execute gnuplot
     my $gnuplot = 'gnuplot';
     $gnuplot = $self->{gnuplot} if (defined $self->{gnuplot});
-    my $err = `$gnuplot $self->{_script} 2>&1`;
+    my $cmd = "$gnuplot $self->{_script}";
+    $cmd .= " -" if ($self->{terminal} =~ /^(ggi|pm|windows|wxt|x11)(\s|$)/);
+    my $err = `$cmd 2>&1`;
 
     # Capture and process error message from Gnuplot
     if (defined $err && $err ne '')
     {
         my ($errTmp) = ($err =~ /\", line \d+:\s(.+)/);
         die "$errTmp\n" if (defined $errTmp);
+        warn "$err\n";
     }
 
     # Convert the image to the user-specified format
-    if (defined $self->{output} && $self->{output} =~ /\./)
+    if (defined $self->{_terminal} && $self->{_terminal} eq 'auto')
     {
         my @a = split(/\./, $self->{output});
         my $ext = $a[-1];
-        if ( $self->{terminal} !~ /^$ext/ &&
-            ($self->{terminal} !~ /^post/ || $ext !~ /^e?ps$/) )
-        {
-            &convert($self, $ext);
-        }
+        &convert($self, $ext) if ($ext !~ /^e?ps$/);
     }
 }
 
@@ -1009,10 +1026,55 @@ sub label
         $out .= " lc $label{pointcolor}" if (defined $label{pointcolor});
     }
 
-    my $pltTmp = $self->{_script};
-    open(PLT, ">>$pltTmp") || confess("Can't write gnuplot script $pltTmp");
-    print PLT "set label $out\n";
-    close(PLT);
+    push(@{$self->{_labels}}, $out);
+    return($self);
+}
+
+
+# Arbitrary arrows placed in the chart
+#
+# Usage example:
+# $chart->arrow(
+#     from      => "0,2",
+#     to        => "0.3,0.1",
+#     linetype  => 'dash',
+#     width     => 2,
+#     color     => "dark-blue",
+#     headsize  => 3,
+#     head      => 'both',
+# );
+sub arrow
+{
+    my ($self, %arrow) = @_;
+    confess("Starting position of arrow not found") if (!defined $arrow{from});
+
+    my $out = '';
+    $out .= " from $arrow{from}";
+    $out .= " to $arrow{to}" if (defined $arrow{to});
+    $out .= " rto $arrow{rto}" if (defined $arrow{rto});
+    $out .= " linetype ".&_lineType($arrow{linetype}) if
+        (defined $arrow{linetype});
+    $out .= " linewidth $arrow{width}" if (defined $arrow{width});
+    $out .= " linecolor rgb \"$arrow{color}\"" if (defined $arrow{color});
+    $out .= " size $arrow{headsize}" if (defined $arrow{headsize});
+
+    if (defined $arrow{head})
+    {
+        if ($arrow{head} eq 'off')
+        {
+            $out .= " nohead";
+        }
+        elsif ($arrow{head} eq 'back')
+        {
+            $out .= " backhead";
+        }
+        elsif ($arrow{head} eq 'both')
+        {
+            $out .= " heads";
+        }
+    }
+
+    push(@{$self->{_arrows}}, $out);
     return($self);
 }
 
@@ -1041,11 +1103,6 @@ sub test
 }
 
 
-sub histogram
-{
-}
-
-
 # Change the image format
 # - called by plot2d()
 #
@@ -1063,37 +1120,39 @@ sub convert
     my $temp = "$self->{_script}.tmp";
     move($self->{output}, $temp);
 
-    if ($self->{terminal} =~ /^post/ && $imgfmt eq 'pdf')
+    # Execute gnuplot
+    my $convert = 'convert';
+    $convert = $self->{convert} if (defined $self->{convert});
+
+    # Rotate 90 deg for landscape image
+    if (defined $self->{orient} && $self->{orient} eq 'portrait')
     {
-        system("ps2pdf $temp $temp".".$imgfmt");
-    }
-    elsif ($self->{terminal} =~ /^pdf/ && $imgfmt eq 'ps')
-    {
-        system("pdf2ps $temp $temp".".$imgfmt");
+        my $cmd = "$convert $temp $temp".".$imgfmt 2>&1";
+        my $err = `$cmd`;
+        if (defined $err && $err ne '')
+        {
+            die "Unsupported image format ($imgfmt)\n" if
+                ($err =~ /^convert: unable to open module file/);
+
+            my ($errTmp) = ($err =~ /^convert: (.+)/);
+            die "$errTmp Perhaps the image format is not supported\n" if
+                (defined $errTmp);
+            die "$err\n";
+        }
     }
     else
     {
-        # Execute gnuplot
-        my $convert = 'convert';
-        $convert = $self->{convert} if (defined $self->{convert});
-
-        # Rotate 90 deg for landscape image
-        if (defined $self->{orient} && $self->{orient} eq 'portrait')
+        my $cmd = "$convert -rotate 90 $temp $temp".".$imgfmt 2>&1";
+        my $err = `$cmd`;
+        if (defined $err && $err ne '')
         {
-            system("$convert $temp $temp".".$imgfmt");
-        }
-        else
-        {
-            my $cmd = "$convert -rotate 90 $temp $temp".".$imgfmt 2>&1";
-            my $err = `$cmd`;
-            if (defined $err && $err ne '')
-            {
-                die "Unsupported image format ($imgfmt)\n" if
-                    ($err =~ /^convert: unable to open module file/);
+            die "Unsupported image format ($imgfmt)\n" if
+                ($err =~ /^convert: unable to open module file/);
 
-                my ($errTmp) = ($err =~ /^convert: (.+)/);
-                die "$errTmp Perhaps the image format is not supported\n";
-            }
+            my ($errTmp) = ($err =~ /^convert: (.+)/);
+            die "$errTmp Perhaps the image format is not supported\n" if
+                (defined $errTmp);
+            die "$err\n";
         }
     }
 
@@ -1173,13 +1232,30 @@ sub pdf
     return($self)
 }
 
+
+# Copy method of the chart object
+sub copy
+{
+    my ($self, $num) = @_;
+    my @clone = &_copy(@_);
+
+    foreach my $clone (@clone)
+    {
+        my $dirTmp = tempdir(CLEANUP => 1);
+        ($^O eq 'MSWin32')? ($dirTmp .= '\\'): ($dirTmp .= '/');
+        $clone->{_script} = $dirTmp . "plot";
+    }
+    return($clone[0]) if (!defined $num);
+    return(@clone);
+}
+
 ################## Chart::Gnuplot::DataSet class ##################
 
 package Chart::Gnuplot::DataSet;
 use strict;
 use Carp;
 use File::Temp qw(tempdir);
-use Chart::Gnuplot::Util qw(_lineType _pointType);
+use Chart::Gnuplot::Util qw(_lineType _pointType _copy);
 
 # Constructor
 sub new
@@ -1207,6 +1283,91 @@ sub AUTOLOAD
 }
 
 
+# xdata get-set method
+sub xdata
+{
+    my ($self, $xdata) = @_;
+    return($self->{xdata}) if (!defined $xdata);
+
+    delete $self->{points};
+    delete $self->{datafile};
+    delete $self->{func};
+    $self->{xdata} = $xdata;
+}
+
+
+# ydata get-set method
+sub ydata
+{
+    my ($self, $ydata) = @_;
+    return($self->{ydata}) if (!defined $ydata);
+
+    delete $self->{points};
+    delete $self->{datafile};
+    delete $self->{func};
+    $self->{ydata} = $ydata;
+}
+
+
+# zdata get-set method
+sub zdata
+{
+    my ($self, $zdata) = @_;
+    return($self->{zdata}) if (!defined $zdata);
+
+    delete $self->{points};
+    delete $self->{datafile};
+    delete $self->{func};
+    $self->{zdata} = $zdata;
+}
+
+
+# points get-set method
+sub points
+{
+    my ($self, $points) = @_;
+    return($self->{points}) if (!defined $points);
+
+    delete $self->{xdata};
+    delete $self->{ydata};
+    delete $self->{zdata};
+    delete $self->{datafile};
+    delete $self->{func};
+    $self->{points} = $points;
+}
+
+
+# datafile get-set method
+sub datafile
+{
+    my ($self, $datafile) = @_;
+    return($self->{datafile}) if (!defined $datafile);
+
+    delete $self->{xdata};
+    delete $self->{ydata};
+    delete $self->{zdata};
+    delete $self->{points};
+    delete $self->{func};
+    $self->{datafile} = $datafile;
+}
+
+
+# func get-set method
+sub func
+{
+    my ($self, $func) = @_;
+    return($self->{func}) if (!defined $func);
+
+    delete $self->{xdata};
+    delete $self->{ydata};
+    delete $self->{zdata};
+    delete $self->{points};
+    delete $self->{datafile};
+    $self->{func} = $func;
+}
+
+
+
 # Thaw the data set object to string
 # - call _fillStyle()
 #
@@ -1529,6 +1690,13 @@ sub _thaw
                 $xLast = $$pt[$i][0];
             }
         }
+        else
+        {
+            for(my $i = 0; $i < @$pt; $i++)
+            {
+                print DATA join(" ", @{$$pt[$i]}), "\n";
+            }
+        }
         close(DATA);
         $string = "'$fileTmp'";
 
@@ -1620,6 +1788,23 @@ sub _fillStyle
 }
 
 
+# Copy method of the data set object
+sub copy
+{
+    my ($self, $num) = @_;
+    my @clone = &_copy(@_);
+
+    foreach my $clone (@clone)
+    {
+        my $dirTmp = tempdir(CLEANUP => 1);
+        ($^O eq 'MSWin32')? ($dirTmp .= '\\'): ($dirTmp .= '/');
+        $clone->{_data} = $dirTmp . "data";
+    }
+    return($clone[0]) if (!defined $num);
+    return(@clone);
+}
+
+
 1;
 
 __END__
@@ -1665,8 +1850,8 @@ Chart::Gnuplot - Plot graph using Gnuplot on the fly
 =head1 DESCRIPTION
 
 This module is to plot graphs uning GNUPLOT on the fly. In order to use this
-module, gnuplot need to be installed. If image format other than PS, PDF and
-EPS is required to generate, the convert program of ImageMagick is also needed.
+module, gnuplot need to be installed. If image format other than PS and EPS is
+required to generate, the convert program of ImageMagick is also needed.
 
 To plot chart using Chart::Gnuplot, a chart object and at least one dataset
 object are required. Information about the chart such as output file, chart
@@ -1909,7 +2094,7 @@ Size (length and height) of the image relative to the default. E.g.
 
 =head3 size
 
-Size of the chart relative to the chart size. This is useful in some
+Size of the plot relative to the chart size. This is useful in some
 multi-plot such as inset chart. E.g.
 
     size => "0.5, 0.4"
@@ -1973,9 +2158,10 @@ have multiple convert executables.
 
 =head3 terminal
 
-The terminal that Gnuplot use. The default terminal is "postscript". This
-attribute is not recommended to be changed unless you are familiar with the
-Gnuplot syntax.  Please test carefully before using this in production code.
+The terminal driver that Gnuplot uses. The default terminal is "postscript".
+This attribute is not recommended to be changed unless you are familiar with
+the Gnuplot syntax. Please check the output image carefully if you use this in
+production code.
 
 Terminal is not necessarily related to the output image format. You may convert
 the image format by the C<convert()> method.
@@ -2041,6 +2227,36 @@ Add an arbitrary text label. e.g.,
         pointcolor => "blue",
     );
 
+=head3 arrow
+
+Draw arbitrary arrow. e.g.,
+
+    $chart->arrow(
+        from      => "0,2",
+        to        => "0.3,0.1",
+        linetype  => 'dash',
+        width     => 2,
+        color     => "dark-blue",
+        headsize  => 3,
+        head      => 'both',
+    );
+
+=head3 copy
+
+Copy the chart object. This method is especially useful when you want to copy a
+chart with highly customized format. E.g.
+
+    my $chart = Chart::Gnuplot->new(
+        ...
+    );
+
+    # $copy and $chart will have the same format
+    my $copy = $chart->copy;
+
+You may also make multiple copies . E.g.
+
+    my @copies = $chart->copy(10);  # make 10 copies
+
 =head3 convert
 
     $chart->convert($imageFmt);
@@ -2315,6 +2531,22 @@ for details.
 Constructor of the dataset object. If no option is specified, default values
 would be used. See L<Dataset Options> for available options.
 
+=head3 copy
+
+Copy the dataset object. This method is especially useful when you want to copy
+a dataset with highly customized format. E.g.
+
+    my $dataset = Chart::Gnuplot::DataSet->new(
+        ...
+    );
+
+    # $copy and $dataset will have the same format and contain the same data
+    my $copy = $dataset->copy;
+
+You may also make multiple copies . E.g.
+
+    my @copies = $dataset->copy(10);  # make 10 copies
+
 =head1 EXAMPLES
 
 Some simple examples are shown below. Many more come with the distribution.
@@ -2478,26 +2710,20 @@ y-axis.
 
 =item 3. Add curve fitting method.
 
-=item 4. Add method to copy Chart and DataSet objects.
+=item 4. Improve the testsuite.
 
-=item 5. Improve the testsuite.
-
-=item 6. Reduce number of temporary files generated.
+=item 5. Reduce number of temporary files generated.
 
 =back
 
 =head1 REQUIREMENTS
 
-L<File::Copy>, L<File::Temp>
+L<File::Copy>, L<File::Temp>, L<Storable>
 
 Gnuplot L<http://www.gnuplot.info>
 
 ImageMagick L<http://www.imagemagick.org> (for full feature)
 
-=head1 TEST ENVIRONMENT
-
-This version is tested against Gnuplot 4.2.0 to 4.2.4 in Linux.
-
 =head1 SEE ALSO
 
 Gnuplot official website L<http://www.gnuplot.info>
@@ -2508,7 +2734,7 @@ Ka-Wai Mak <kwmak@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008-2009 Ka-Wai Mak. All rights reserved.
+Copyright (c) 2008-2010 Ka-Wai Mak. All rights reserved.
 
 =head1 LICENSE
 
